@@ -24,15 +24,23 @@ index_seq <- function(x) {
 }
 
 
-
-is_linear <- function(x, tolerance) {
-  x_seq <- index_seq(x)
-  x_seq <- dplyr::near(x_seq, min(x_seq), tol = tolerance)
-  all(x_seq)
+is_seq_linear_basic <- function(x) {
+  if (length(x) < 3L) {
+    return(TRUE)
+  }
+  # As the difference between each successive pair of values must be equal for
+  # `x` to be a linear sequence, we can take the first pairwise difference and
+  # test each other difference for equality with it. If any comparison turns out
+  # unequal, `x` is not a linear sequence.
+  diff_first <- x[2L] - x[1L]
+  for (i in 3L:length(x)) {
+    if (x[i] - x[i - 1L] != diff_first) {
+      return(FALSE)
+    }
+  }
+  TRUE
 }
 
-
-
 is_seq_ascending_basic <- function(x) {
   for (i in 1L:(length(x) - 1L)) {
     if (x[i + 1L] <= x[i]) {
@@ -53,6 +61,14 @@ is_seq_descending_basic <- function(x) {
 }
 
 
+# # Test any of the sequence functions interactively:
+# x <- c(1, 2, NA, 4)
+# tolerance <- .Machine$double.eps^0.5
+# test_linear <- TRUE
+# test_special <- NULL
+# min_length <- NULL
+# args_other <- NULL
+
 
 # Non-exported workhorse API of all the sequence predicates:
 is_seq_basic <- function(x, tolerance = .Machine$double.eps^0.5,
@@ -69,13 +85,6 @@ is_seq_basic <- function(x, tolerance = .Machine$double.eps^0.5,
     if (is_even(length(x))) {
       return(FALSE)
     }
-    # Save the unmodified `x` for a test that is conducted if `x` contains one
-    # or more `NA` elements:
-    x_orig <- x
-  }
-
-  if (all(is.na(x))) {
-    return(NA)
   }
 
   if (!is_numeric_like(x)) {
@@ -90,31 +99,66 @@ is_seq_basic <- function(x, tolerance = .Machine$double.eps^0.5,
     return(TRUE)
   }
 
-  x_has_na <- any(is.na(x))
+  x_has_na <- anyNA(x)
 
   if (x_has_na) {
 
-    # These two `while`-loops remove all `NA` values from the start and the end
-    # of `x` because `NA`s at these particular locations have no bearing on
-    # whether or not `x` might represent a linear sequence:
-    while (is.na(x[1L])) {
-      x <- x[-1L]
+    # Save the unmodified `x` for a test that is conducted if `x` contains one
+    # or more `NA` elements:
+    x_orig <- x
+    n_x_orig <- length(x)
+
+    not_na <- which(!is.na(x))
+
+    # Need at least three known values:
+    if (length(not_na) < 3L) {
+      return(NA)
     }
-    while (is.na(x[length(x)])) {
-      x <- x[-length(x)]
+
+    # # Indices of `NA`s at the start and end of `x`:
+    # n_na_start <- seq_len(not_na[1L] - 1L)
+    # n_na_end   <- (not_na[length(not_na)] + 1L):n_x_orig
+
+    n_na_start <- match(FALSE,     is.na(x_orig))  - 1L
+    n_na_end   <- match(FALSE, rev(is.na(x_orig))) - 1L
+
+    # Remove all `NA` values from the start and the end of `x` because `NA`s at
+    # these particular locations cannot disprove that `x` is the kind of
+    # sequence of interest. (They do mean that it cannot be proven, so the
+    # function will return either `NA` or `FALSE`, depending on other factors.)
+    x <- x[not_na[1L]:not_na[length(not_na)]]
+
+    if (test_linear && !anyNA(x) && !is_seq_linear_basic(x)) {
+      return(FALSE)
     }
 
     # If the removal of leading and / or trailing `NA` elements in `x` caused
     # the central value to shift, or if only one side from among left and right
-    # had any `NA` values, the original `x` was not symmetrically grouped around
-    # that value, and hence not a dispersed sequence:
-    if (!is.null(test_special) && test_special == "dispersed") {
-      diff_central_index <-
-        !dplyr::near(args_other$from, x[index_central(x)], tolerance)
-      one_sided_na <- (!is.na(x_orig[1L])) || (!is.na(x_orig[length(x_orig)]))
-      if (is_even(length(x)) || diff_central_index || one_sided_na) {
-        return(FALSE)
+    # had any `NA` values, the original `x` might not have been symmetrically
+    # grouped around that value, and hence not a dispersed sequence. It depends
+    # on whether it could potentially be dispersed, assuming values consistent
+    # with this idea that are imputed for the `NA`s here.
+    if (
+      !is.null(test_special) &&
+      test_special == "dispersed" &&
+      n_na_start != n_na_end
+    ) {
+      unit <- x[2] - x[1]
+
+      # Impute sequences of regularly spaces values that the `NA`s might stand
+      # for, so that the input could potentially be dispersed around
+      # `args_other$from`...
+      seq_na_start <- seq(x[1],         length.out = n_na_start, by = -unit) - 1
+      seq_na_end   <- seq(x[length(x)], length.out = n_na_end,   by =  unit) + 1
+
+      seq_imputed <- c(rev(seq_na_start), x, seq_na_end)
+
+      # ...assuming the overall (partly imputed) sequence is still dispersed
+      # around that value:
+      if (is_seq_dispersed(seq_imputed, from = args_other$from)) {
+        return(NA)
       }
+      return(FALSE)
     }
 
     # Used within the for loop below to check whether the step size must be
@@ -156,7 +200,8 @@ is_seq_basic <- function(x, tolerance = .Machine$double.eps^0.5,
           # elements, which invariably means that the numbers surrounding the
           # `NA`s are too far spaced out for there to be a linear sequence. In
           # either case...
-          seq_replacement_has_wrong_length <- length(seq_replacement) == 0L ||
+          seq_replacement_has_wrong_length <-
+            length(seq_replacement) == 0L ||
             length(seq_replacement) > length(index_lower:index_upper)
 
           # ...an error is thrown:
@@ -181,13 +226,10 @@ is_seq_basic <- function(x, tolerance = .Machine$double.eps^0.5,
   # in the for loop above -- for linearity:
   if (test_linear) {
     x_seq <- index_seq(x)
-    x_seq <- dplyr::near(x_seq, min(x_seq), tol = tolerance)
-    pass_test <- all(x_seq)
-    if (!pass_test) {
+    pass_test_linear <- all(dplyr::near(x_seq, min(x_seq), tol = tolerance))
+    if (!pass_test_linear) {
       return(FALSE)
     }
-  } else {
-    pass_test <- TRUE
   }
 
   # Interface for the special variant functions:
@@ -197,12 +239,13 @@ is_seq_basic <- function(x, tolerance = .Machine$double.eps^0.5,
       "ascending"  = is_seq_ascending_basic(x),
       "descending" = is_seq_descending_basic(x),
       "dispersed"  = is_seq_dispersed_basic(x, args_other$from, tolerance)
+      # TODO: MAKE THIS "DISPERSED" TEST ABLE TO ASSUME THAT `NA`S IN `x_orig`
+      # ARE ACTUALLY DISPERSED VALUES! MAYBE USE `index_central()`'S INTERNAL
+      # LOGIC AND BUILD UP ON IT.
     )
-    pass_test <- pass_test && pass_test_special
-  }
-
-  if (!pass_test) {
-    return(FALSE)
+    if (!pass_test_special) {
+      return(FALSE)
+    }
   }
 
   if (x_has_na) {
@@ -252,7 +295,7 @@ is_seq_basic <- function(x, tolerance = .Machine$double.eps^0.5,
 #'   sequence after the `NA`s were replaced by appropriate values? If so, they
 #'   return `NA`; otherwise, they return `FALSE`.
 
-#' @seealso [`validate::is_linear_sequence()`], which is much like
+#' @seealso `validate::is_linear_sequence()`, which is much like
 #'   `is_seq_linear()` but more permissive with `NA` values. It comes with some
 #'   additional features, such as support for date-times.
 
